"""add hide_unless_approved column

Revision ID: 7a3a4755b4a8
Revises: 1040ee4836af
Create Date: 2017-08-09 21:53:01.995660

"""

# revision identifiers, used by Alembic.
revision = "7a3a4755b4a8"
down_revision = "1040ee4836af"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        "Namespace", sa.Column("hide_unless_approved", sa.Boolean(), nullable=True)
    )
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("Namespace", "hide_unless_approved")
    ### end Alembic commands ###
