"""namespace ignore_query_string

Revision ID: 051f8b810f26
Revises: 3ee195d142fb
Create Date: 2019-02-20 10:27:29.804965

"""

# revision identifiers, used by Alembic.
revision = '051f8b810f26'
down_revision = '3ee195d142fb'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    from sqlalchemy.sql import expression
    op.add_column('rb_namespace', sa.Column('ignore_query_string', sa.Boolean(), nullable=True, server_default=expression.false()))


def downgrade():
    pass
