"""namespace reverse node order

Revision ID: f7b42b2024fb
Revises: 40ded1163f95
Create Date: 2019-03-19 11:17:31.427157

"""

# revision identifiers, used by Alembic.
revision = 'f7b42b2024fb'
down_revision = '40ded1163f95'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('rb_namespace', sa.Column('reverse_order', sa.Boolean(), nullable=True))

def downgrade():
    pass
