  <link rel="stylesheet" href="/static/css/pygments.css">
  <link rel="stylesheet" href="/static/css/common.css">
  <link rel="stylesheet" href="/dynamic-remarkbox.css">

{%- if request.stylesheet_uri %}
  <link rel="stylesheet" href="{{ request.stylesheet_uri }}">
{%- endif %}

{%- if request.mode == 'embed' %}
  <link rel="stylesheet" href="/static/css/embed.css">
  {% if request.stylesheet_embed %}
  <link rel="stylesheet" href="/embed/ns/{{ request.namespace.name }}/{{ request.namespace.name }}.{{ request.namespace.stylesheet_embed_timestamp }}.css">
  {% endif %}
{%- else %}
  {% if request.stylesheet %}
  <link rel="stylesheet" href="/ns/{{ request.namespace.name }}/{{ request.namespace.name }}.{{ request.namespace.stylesheet_timestamp }}.css">
  {% endif %}
{%- endif %}
