  {% if request.namespace.hide_powered_by %}
  {% else %}
  <br/>
  <br/>
  <p class="powered-by">
    <a href="http://www.remarkbox.com" rel="nofollow" target="_blank" title="Powered by Remarkbox"><img src="/static/img/remarkbox-logo.png" width="24px" style="vertical-align: middle;"> &nbsp; &nbsp; Remarkbox</a>
  </p>
  <br/>
  {% endif %}
