"""autogenerated indicies.

Revision ID: 04e52337756e
Revises: 6a440d3921d2
Create Date: 2017-07-03 13:45:32.742934

"""

# revision identifiers, used by Alembic.
revision = "04e52337756e"
down_revision = "6a440d3921d2"
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.create_index(op.f("ix_Node_id"), "Node", ["id"], unique=False)
    op.create_index(op.f("ix_Node_root_id"), "Node", ["root_id"], unique=False)
    op.create_index(op.f("ix_Uri_data"), "Uri", ["data"], unique=False)
    op.create_index(op.f("ix_Uri_id"), "Uri", ["id"], unique=False)
    op.create_index(op.f("ix_User_id"), "User", ["id"], unique=False)
    op.create_index(op.f("ix_Vote_id"), "Vote", ["id"], unique=False)
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_index(op.f("ix_Vote_id"), table_name="Vote")
    op.drop_index(op.f("ix_User_id"), table_name="User")
    op.drop_index(op.f("ix_Uri_id"), table_name="Uri")
    op.drop_index(op.f("ix_Uri_data"), table_name="Uri")
    op.drop_index(op.f("ix_Node_root_id"), table_name="Node")
    op.drop_index(op.f("ix_Node_id"), table_name="Node")
    ### end Alembic commands ###
