"""new field for namespace request cache

Revision ID: 14a6a35940c7
Revises: 8f76e372a953
Create Date: 2024-10-06 09:17:13.978089

"""

# revision identifiers, used by Alembic.
revision = '14a6a35940c7'
down_revision = '8f76e372a953'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa



from sqlalchemy_utils import UUIDType as TempUUIDType
UUIDType = TempUUIDType(binary=False)


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('rb_namespace_request', sa.Column('last_scrape_timestamp', sa.BigInteger(), nullable=True))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('rb_namespace_request', 'last_scrape_timestamp')
    # ### end Alembic commands ###
