<?xml version="1.0" encoding="UTF-8"?>
<urlset xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">
{% for node in nodes %}
    <url>
        <loc>{{ request.host_url }}{{ node.path }}</loc>
        <changefreq>weekly</changefreq>
        <priority>0.8</priority>
	<lastmod>{{ node.changed_datetime.isoformat() }}</lastmod>
    </url>
{% endfor %}
</urlset>
