"""password attempts field

Revision ID: 8f76e372a953
Revises: b132ec968f0c
Create Date: 2022-06-20 10:31:36.219490

"""

# revision identifiers, used by Alembic.
revision = '8f76e372a953'
down_revision = 'b132ec968f0c'
branch_labels = None
depends_on = None

from alembic import op
import sqlalchemy as sa


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('rb_user', sa.Column('password_attempts', sa.INTEGER(), nullable=False, server_default="0"))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('rb_user', 'password_attempts')
    # ### end Alembic commands ###
