{% if 'slack.public' in request.app %}

<a href="https://slack.com/oauth/authorize?scope={{ request.app.get('slack.oauth_scope') | urlencode }}&client_id={{ request.app['slack.public'] }}&redirect_uri={{ request.route_url('oauth-slack') }}?namespace={{ request.namespace.name }}">
  <img alt="Add to Slack" height="40" width="139" src="https://platform.slack-edge.com/img/add_to_slack.png" srcset="https://platform.slack-edge.com/img/add_to_slack.png 1x, https://platform.slack-edge.com/img/add_to_slack@2x.png 2x" />
</a>

{% endif %}
