<?xml version="1.0"?>
<!-- RSS generated by Remarkbox (https://www.remarkbox.com) -->
<rss version="2.0" xmlns:{{ request.namespace.name }}="https://{{ request.app_domain }}/ns/{{ request.namespace.name }}">
	<channel>
	{% if nodes %}
		{% set feed_type = "Nodes" %}
		{% if nodes[0].is_root %}{% set feed_type = "Threads" %}{% endif %}
		<title>{{ feed_type }}: {{ request.namespace.name }}</title>
		<link>https://{{ request.app_domain }}/ns/{{ request.namespace.name }}</link>
		<description>{{ feed_type }} for {{ request.namespace.name }}</description>
		<language>en-us</language>
		<lastBuildDate>{{ nodes[0].changed_datetime }}</lastBuildDate>
		<docs>https://faq.remarkbox.com</docs>
		<ttl>20</ttl>

		{% for node in nodes %}
		<item>
			{% if node.title %}
			<title>{{ node.title }}</title>
			{% endif %}
			<description>
			{% if node.is_root and node.title %}
			{{ node.title }}
			{% elif node.is_root and node.has_uri %}
			{{ node.uri.data }}
			{% else %}
			{{ node.data }}
			{% endif %}
			</description>
			<pubDate>{{ node.changed_datetime }}</pubDate>
			<guid>https://{{ request.app_domain }}/r/{{ node.id }}</guid>
			<link>https://{{ request.app_domain }}/r/{{ node.id }}</link>
			<comments>https://{{ request.app_domain }}/r/{{ node.id }}</comments>
			<author>{% if node.is_verified %}{{ node.user.name }}{% else %}unverified{% endif %}</author>
		</item>
		{% endfor %}

	{% endif %}
	</channel>
</rss>
